import unittest

from base_test_case import BaseTestCase

class SparkContextTestCase(BaseTestCase):
    def test_set_log_level(self):
        self.sc.setLogLevel('ERROR')

    def test_get_hadoop_config(self):
        self.sc._jsc.hadoopConfiguration().set('foo', 'bar')
        val = self.sc._jsc.hadoopConfiguration().get('foo')

        self.assertEqual(val, 'bar')

if __name__ == '__main__':
    unittest.main()



import sys
import os
import shutil
import unittest
import pandas

from base_test_case import BaseTestCase

from pyspark_proxy.server.capture import Capture

class DataFrameTestCase(BaseTestCase):
    def setUp(self):
        self.current_path = os.path.abspath(os.path.dirname(__file__))
        self.output_path = os.path.join(self.current_path, 'test_output')

        self.df = self.sqlContext.read.json(os.path.join(self.current_path, 'data', 'data.json'))

    def tearDown(self):
        if os.path.exists(self.output_path):
            shutil.rmtree(self.output_path)

    def test_count(self):
        self.assertEqual(self.df.count(), 3)

    def test_temp_table(self):
        self.df.createOrReplaceTempView('my_table')

        results = self.sqlContext.sql('select count(*) from my_table')

        self.assertEqual(results.count(), 1)

    def test_show(self):
        with Capture() as output:
            self.df.show()

        expected_output = [
            u'+---+---+',
            u'|bar|foo|',
            u'+---+---+',
            u'|  2|  1|',
            u'|  4|  3|',
            u'|  6|  5|',
            u'+---+---+',
            u'']

        self.assertEqual(expected_output, output)

        with Capture() as output:
            self.df.show(1)

        expected_output = [
            u'+---+---+',
            u'|bar|foo|',
            u'+---+---+',
            u'|  2|  1|',
            u'+---+---+',
            u'only showing top 1 row',
            u'']

        self.assertEqual(expected_output, output)

    def test_to_pandas(self):
        pdf = self.df.toPandas()

        self.assertEqual(type(pdf), pandas.core.frame.DataFrame)

    def test_collect(self):
        rows = self.df.collect()

        self.assertEqual(rows[0]['foo'], 1)

    def test_columns(self):
        cols = self.df.columns

        self.assertEqual(cols, ['bar', 'foo'])

    def test_dtypes(self):
        dtypes = self.df.dtypes

        self.assertEqual(dtypes, [[u'bar', u'bigint'], [u'foo', u'bigint']])

if __name__ == '__main__':
    unittest.main()



import unittest

from base_test_case import BaseTestCase

class ExceptionTestCase(BaseTestCase):
    def test_exception(self):
        try:
            df = self.sqlContext.read.json('/tmp/unknown/path.json')
        except Exception as e:
            self.assertIn('Path does not exist', str(e))

if __name__ == '__main__':
    unittest.main()



import sys
from pyspark_proxy.proxy import Proxy

__all__ = ['SparkContext']

class SparkContext(Proxy):
    # _jsc is actually a java object with no python counterpart
    # cant be as dynamic as some of the other classes/functions so
    # the actual function calls are scoped out a bit more
    @property
    def _jsc(self):
        self._func_chain = [{'func': '_jsc'}]

        class Jsc():
            def hadoopConfiguration(jsc_self):
                self._func_chain.append({'func': 'hadoopConfiguration', 'args': (), 'kwargs': {}})

                return jsc_self

            def get(jsc_self, *args):
                self._func_chain.append({'func': 'get', 'args': args, 'kwargs': {}})
                return self._call_chain(self._id)

            def set(jsc_self, *args):
                self._func_chain.append({'func': 'set', 'args': args, 'kwargs': {}})
                return self._call_chain(self._id)

        return Jsc()



from pyspark_proxy.proxy import Proxy

from pyspark_proxy.sql.readwriter import DataFrameReader

__all__ = ['SQLContext']

class SQLContext(Proxy):
    @classmethod
    def getOrCreate(cls, *args, **kwargs):
        res = cls._call_class_method('getOrCreate', (args, kwargs))

        ctxt = SQLContext(no_init=True)
        ctxt._id = res['id']

        return ctxt

    @property
    def read(self):
        return DataFrameReader(self._id, 'read')



import os
import shutil
import unittest

from base_test_case import BaseTestCase

class DataFrameWriterTestCase(BaseTestCase):
    def setUp(self):
        self.output_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'test_output')
        self.json_output = os.path.join(self.output_path, 'output.json')
        self.csv_output = os.path.join(self.output_path, 'csv.json')

        data = [(1,2,'a'),(3,4,'b'),(5,6,'c')]
        self.df = self.sqlContext.createDataFrame(data, ['foo', 'bar', 'baz'])

    def tearDown(self):
        if os.path.exists(self.output_path):
            shutil.rmtree(self.output_path)

    def test_json(self):
        self.df.write.json(self.json_output)

        self.assertTrue(os.path.exists(self.json_output))

    def test_csv(self):
        self.df.write.csv(self.csv_output)

        self.assertTrue(os.path.exists(self.csv_output))

    def test_parquet(self):
        path = os.path.join(self.output_path, 'output.parquet')
        self.df.write.parquet(path)

        self.assertTrue(os.path.exists(os.path.join(path, '_SUCCESS')))

    def test_text(self):
        path = os.path.join(self.output_path, 'output.txt')
        self.df.select('baz').write.text(path)

        self.assertTrue(os.path.exists(path))

    def test_orc(self):
        path = os.path.join(self.output_path, 'output.orc')
        self.df.write.orc(path)

        self.assertTrue(os.path.exists(os.path.join(path, '_SUCCESS')))

    def test_mode(self):
        self.df.write.json(self.json_output)
        self.df.write.mode('overwrite').json(self.json_output)

        self.assertTrue(os.path.exists(self.json_output))

    def test_format(self):
        self.df.write.format('json').save(self.json_output)

        self.assertTrue(os.path.exists(self.json_output))

    def test_option(self):
        self.df.write.option('header', True).csv(self.csv_output)

        self.assertTrue(os.path.exists(self.csv_output))

    def test_options(self):
        self.df.write.options(header=True).csv(self.csv_output)

        self.assertTrue(os.path.exists(self.csv_output))

    def test_partitionby(self):
        self.df.write.partitionBy('baz').json(self.json_output)

        self.assertTrue(os.path.exists(os.path.join(self.json_output, '_SUCCESS')))
        self.assertTrue(os.path.exists(os.path.join(self.json_output, 'baz=a')))
        self.assertTrue(os.path.exists(os.path.join(self.json_output, 'baz=b')))
        self.assertTrue(os.path.exists(os.path.join(self.json_output, 'baz=c')))

    def test_partitionby(self):
        self.df.write.format('parquet').bucketBy(3, 'baz').saveAsTable('bucketed_table')
        df = self.sqlContext.sql('select * from bucketed_table')

        self.assertEquals(df.count(), 3)

    def test_sortby(self):
        self.df.write.format('parquet')\
            .bucketBy(3, 'baz')\
            .sortBy('foo')\
            .saveAsTable('sorted_table')
        df = self.sqlContext.sql('select * from sorted_table')

        self.assertEquals(df.count(), 3)

    def test_save(self):
        self.df.write.save(self.json_output, format='json')

        self.assertTrue(os.path.exists(self.json_output))

    def test_insert_into(self):
        self.df.write.format('parquet').saveAsTable('temp_table')

        data = [(1,2,'a'),(3,4,'b'),(5,6,'c')]
        df = self.sqlContext.createDataFrame(data, ['foo', 'bar', 'baz'])

        df.write.insertInto('temp_table')

        inserted_df = self.sqlContext.table('temp_table')

        self.assertEqual(inserted_df.count(), 6)

    def test_save_as_table(self):
        self.df.write.saveAsTable('save_as_table', format='parquet')
        df = self.sqlContext.table('save_as_table')

        self.assertEqual(df.count(), 3)

    def test_multiple_writes(self):
        self.df.write.json(self.json_output)
        self.df.write.csv(self.csv_output)

        self.assertTrue(os.path.exists(self.json_output))
        self.assertTrue(os.path.exists(self.csv_output))

if __name__ == '__main__':
    unittest.main()



from pyspark_proxy.proxy import Proxy

__all__ = ['Column']

class Column(Proxy):
    def __init__(self, id, dfid, name):
        self._dfid = dfid
        self._name = name
        self._id = id

    def alias(self, *args, **kwargs):
        res = self._call(self._id, 'alias', (args, kwargs))

        return Column(res['id'], self._dfid, args[0])

    def cast(self, *args, **kwargs):
        res = self._call(self._id, 'cast', (args, kwargs))

        return Column(res['id'], self._dfid, self._name)



import subprocess
import unittest
from multiprocessing import Process
import time

import requests

from pyspark_proxy import SparkContext
from pyspark_proxy.sql import SQLContext

import pyspark_proxy.server as server

class BaseTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = Process(target=server.run, kwargs={'debug': True, 'use_reloader': False})
        cls.server.start()
        cls.server.join(1) # needs some time to boot up the webserver

        cls.sc = SparkContext(appName='test_case')
        cls.sc.setLogLevel('ERROR')
        cls.sqlContext = SQLContext(cls.sc)

    @classmethod
    def tearDownClass(self):
        #since the tests are being run in a spark application the
        #spark context needs to be manually killed since killing the proc
        #doesn't kill the contexts created
        requests.get('http://localhost:5000/clear')

        self.server.terminate()



import sys
import uuid
import json
import pickle
import base64

import requests

class Proxy(object):
    _PROXY = True
    _id = None
    _args = None
    _kwargs = None
    _module = None
    _class = None
    _func_chain = []

    def __init__(self, *args, **kwargs):
        self._id = str(uuid.uuid4())
        self._kwargs = kwargs
        self._class = self.__class__.__name__
        self._args = args
        self._module = sys.modules[self.__class__.__module__].__name__.replace('pyspark_proxy', 'pyspark')

        if 'no_init' not in kwargs:
            self._create_object()

        # pickled objects returned from the server can have the pyspark path
        # module instead of pyspark_proxy module. this creates an alias for
        # pyspark to point to pyspark_proxy so hopefully pickleable objects will
        # use the pyspark_proxy version of the object.
        if 'pyspark' not in sys.modules:
            sys.modules['pyspark'] = sys.modules['pyspark_proxy']

    def _create_object(self):
        args = []

        for x in self._args:
            if hasattr(x, '_PROXY'):
                args.append({'_PROXY_ID': x._id})
            else:
                args.append(x)

        body = {
                'module': self._module,
                'class': self._class,
                'kwargs': self._kwargs,
                'args': args,
                'id': self._id
                }

        r = requests.post('http://localhost:5000/create', json=body)

    # for a single function call
    # ex: df.write.csv('foo.csv')
    # ex: df.count()
    def _call(self, base_obj, path, function_args):
        args = []

        for x in function_args[0]:
            if hasattr(x, '_PROXY'):
                args.append({'_PROXY_ID': x._id})
            else:
                args.append(x)

        body = {
                'id': base_obj,
                'path': path,
                'args': args,
                'kwargs': function_args[1]
                }

        r = requests.post('http://localhost:5000/call', json=body)
        res_json = r.json()
        
        return self._handle_response(res_json)

    # for chained function calls
    # ex: df.format('json').save('bar.json')
    # 
    # currently any implemented function doesn't return an object
    # if in a later time _call_chain needs to return an object, might
    # merge this with the regular _call function
    def _call_chain(self, base_obj):
        body = {
            'id': base_obj,
            'stack': self._func_chain
            }

        r = requests.post('http://localhost:5000/call_chain', json=body)
        res_json = r.json()

        self._func_chain = []

        return self._handle_response(res_json)

    # for class function calls
    # ex: SQLContext.getOrCreate(spark_context)
    #
    # This might need to be reworked once more class methods are
    # implemented and there is a better understanding of requirements
    @classmethod
    def _call_class_method(cls, function, function_args):
        args = []

        for x in function_args[0]:
            if hasattr(x, '_PROXY'):
                args.append({'_PROXY_ID': x._id})
            else:
                args.append(x)

        body = {
            'class': cls.__name__,
            'module': cls.__module__.replace('pyspark_proxy', 'pyspark'),
            'function': function,
            'args': args,
            'kwargs': function_args[1]
        }

        r = requests.post('http://localhost:5000/call_class_method', json=body)
        res_json = r.json()

        return res_json

    # __getitem__ server call 
    # ex: df['age']
    def _get_item(self, item):
        body = {
            'id': self._id,
            'item': item
            }

        r = requests.get('http://localhost:5000/get_item', json=body)
        return r.json()

    # parses the response json from the server and returns the proper object
    def _handle_response(self, resp):
        if resp['stdout'] != []:
            print('\n'.join(resp['stdout']))

        if resp['exception']:
            raise Exception(resp['exception'])

        if resp['object']:
            if 'id' in resp:
                if resp['class'] == 'DataFrame':
                    from pyspark_proxy.sql.dataframe import DataFrame

                    return DataFrame(resp['id'])
                else:
                    return resp
            elif 'pickle' == resp['class']:
                return pickle.loads(base64.b64decode(resp['value']))
            else:
                return resp['value']
        else:
            return None

    # catch all function
    def __getattr__(self, name):
        def method(*args, **kwargs):
            return self._call(self._id, name, (args, kwargs))

        return method



import os
import unittest

from base_test_case import BaseTestCase

class DataFrameReaderTestCase(BaseTestCase):
    def setUp(self):
        self.data_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'data')
        self.json_path = os.path.join(self.data_path, 'data.json')
        self.csv_path = os.path.join(self.data_path, 'data.csv')

    def test_read_json(self):
        df = self.sqlContext.read.json(self.json_path)

        self.assertEqual(df.count(), 3)

    def test_read_csv(self):
        df = self.sqlContext.read.csv(self.csv_path, header=True)

        self.assertEqual(df.count(), 3)

    def test_read_text(self):
        df = self.sqlContext.read.text(os.path.join(self.data_path, 'data.txt'))

        self.assertEqual(df.count(), 3)

    def test_read_parquet(self):
        df = self.sqlContext.read.parquet(os.path.join(self.data_path, 'data.parquet'))

        self.assertEqual(df.count(), 3)

    def test_read_orc(self):
        df = self.sqlContext.read.orc(os.path.join(self.data_path, 'data.orc'))

        self.assertEqual(df.count(), 3)

    def test_option(self):
        df = self.sqlContext.read.option('header', True).csv(self.csv_path)

        self.assertEqual(df.count(), 3)

    def test_options(self):
        df = self.sqlContext.read.options(header=True).csv(self.csv_path)

        self.assertEqual(df.count(), 3)

    def test_format(self):
        df = self.sqlContext.read.format('json').load(self.json_path)

        self.assertEqual(df.count(), 3)

    def test_load(self):
        df = self.sqlContext.read.load(self.json_path, format='json')

        self.assertEqual(df.count(), 3)

    def test_load_multiple_paths(self):
        df = self.sqlContext.read.format('json')\
            .load([self.json_path, self.json_path])

        self.assertEqual(df.count(), 6)

    def test_table(self):
        df = self.sqlContext.read.parquet(os.path.join(self.data_path, 'data.parquet'))
        df.createOrReplaceTempView('tmpTable')

        df_table = self.sqlContext.read.table('tmpTable')
        self.assertEqual(df_table.count(), 3)

    def test_schema(self):
        s = self.sqlContext.read.schema("foo STRING, bar INT")
        df = s.json(self.json_path)
        df_types = df.dtypes

        self.assertEqual(df_types[0][1], 'string')
        self.assertEqual(df_types[1][1], 'int')

if __name__ == '__main__':
    unittest.main()



from pyspark_proxy.proxy import Proxy

from pyspark_proxy.sql.readwriter import DataFrameWriter
from pyspark_proxy.sql.column import Column

__all__ = ['DataFrame']

class DataFrame(Proxy):
    _functions = [
            'toJSON', 'registerTempTable', 'createTempView', 'createOrReplaceTempView',
            'createGlobalTempView', 'createOrReplaceGlobalTempView', 'printSchema',
            'explain', 'exceptAll', 'isLocal', 'show', 'checkpoint', 'localCheckpoint',
            'withWatermark', 'hint', 'count', 'collect', 'toLocalIterator', 'limit',
            'take', 'foreach', 'foreachPartition', 'cache', 'persist', 'storageLevel',
            'unpersist', 'coalesce', 'repartition', 'distinct', 'sample', 'samplyBy',
            'randomSplit', 'colRegex', 'alias', 'crossJoin', 'join', 'sortWithinPartitions',
            'sort', 'describe', 'summary', 'head', 'first', 'select', 'selectExpr',
            'filter', 'groupBy', 'rollup', 'cube', 'agg', 'union', 'unionAll',
            'unionByName', 'intersect', 'subtract', 'dropDuplicates', 'dropna', 'fillna',
            'replace', 'approxQuantile', 'corr', 'cov', 'crosstab', 'feqItems',
            'withColumn', 'withColumnRenamed', 'drop', 'toDF', 'toPandas']

    _column_obj = {}
    _columns = []

    def __init__(self, id):
        self._id = id

    @property
    def dtypes(self):
        return self._call(self._id, 'dtypes', ((), {}))

    @property
    def columns(self):
        if self._columns == []:
            self._columns = self._call(self._id, 'columns', ((), {}))

        return self._columns

    @property
    def write(self):
        return DataFrameWriter(self._id, 'write')

    # retrieves column by key lookup
    # df['age']
    def __getitem__(self, item):
        if item not in self.columns:
            raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, item))
        else:
            if item not in self._column_obj:
                res = self._get_item(item)
                col = Column(res['id'], self._id, item)
            else:
                col = self._column_obj[item]

            return col

    # retrieves column by attribute
    # df.age
    def __getattr__(self, item):
        if item in self._functions:
            def method(*args, **kwargs):
                return self._call(self._id, item, (args, kwargs))

            return method
        else:
            return self[item]



import uuid
import os
import pickle
import base64
import logging

logger = logging.getLogger()

import findspark
findspark.init()

import pyspark
from flask import Flask, request, jsonify
from flask.logging import default_handler

from pyspark_proxy.server.capture import Capture

app = Flask(__name__)

objects = {}

# looks at incomming arguments to see if there are any
# pyspark objects that should be retrieved and passed in
def arg_objects(request_args):
    global objects

    args = []

    for a in request_args:
        if type(a) == dict and '_PROXY_ID' in a:
            id = a['_PROXY_ID']
            print('Retrieving object id: %s' % id)

            args.append(objects[id])
        else:
            args.append(a)

    return args

# checks if any objects are created via a function call
# and returns the corresponding object
#
# pandas: pickles and returns a pandas dataframe
# pyspark.*: returns class and corresponding object id
def object_response(obj, exception, paths=[], stdout=[]):
    global objects

    result = {
            'object': False,
            'stdout': stdout,
            'exception': exception
            }

    if obj is not None:
        result['object'] = True
        result['class'] = obj.__class__.__name__

        if 'pyspark' in str(obj.__class__):
            id = str(uuid.uuid4())

            result['id'] = id

            logger.info('Adding object id %s to the stack' % id)
            objects[id] = obj
        # the last string in paths is the function that gets called
        elif paths[-1] == 'toPandas' or paths[-1] == 'collect':
            result['class'] = 'pickle'
            result['value'] = base64.b64encode(pickle.dumps(obj, 2))
        else:
            result['value'] = obj

    return result

@app.route('/create', methods=['POST'])
def create():
    global objects
    
    req = request.json

    logger.info('/create')
    logger.info(req)

    module_paths = req['module'].split('.')
    base_module = __import__(module_paths[0])

    module = base_module

    # __import__ only imports the top level module
    # use loop through the rest via getattr to get the 
    # last module with the needed object
    if len(module_paths) > 1:
        for m in module_paths[1:]:
            module = getattr(module, m)

    callable = getattr(module, req['class'])
    objects[req['id']] = callable(*arg_objects(req['args']), **req['kwargs'])

    return req['id']

@app.route('/call', methods=['POST'])
def call():
    global objects
    
    req = request.json

    logger.info('/call')
    logger.info(req)

    result_exception = None
    base_obj = objects[req['id']]
    paths = req['path'].split('.')

    func = base_obj

    try:
        for p in paths:
            func = getattr(func, p)
        
        with Capture() as stdout:

            if callable(func):
                res_obj = func(*arg_objects(req['args']), **req['kwargs'])
            else:
                res_obj = func
    except Exception as e:
        result_exception = str(e)

    return jsonify(object_response(res_obj, result_exception, paths, stdout))

@app.route('/call_chain', methods=['POST'])
def call_chain():
    global objects

    req = request.json

    logger.info('/call_chain')
    logger.info(req)

    base_obj = objects[req['id']]

    obj = base_obj
    res_obj = None
    result_exception = None

    try:
        with Capture() as stdout:
            for s in req['stack']:
                obj = getattr(obj, s['func'])

                if callable(obj):
                    obj = obj(*s['args'], **s['kwargs'])

                    # only check for returned objects for the
                    # last call in the chain
                    if s == req['stack'][-1]:
                        res_obj = obj
    except Exception as e:
        result_exception = str(e)

    return jsonify(object_response(res_obj, result_exception, req['stack'], stdout))

@app.route('/call_class_method', methods=['POST'])
def call_class_method():
    global objects

    req = request.json

    logger.info('/call_call_method')
    logger.info(req)

    res_obj = None
    result_exception = None

    module_paths = req['module'].split('.')
    base_module = __import__(module_paths[0])

    module = base_module

    if len(module_paths) > 1:
        for m in module_paths[1:]:
            module = getattr(module, m)

    callable = getattr(module, req['class'])

    # should this capture stdout?
    try:
        res_obj = callable(*arg_objects(req['args']), **req['kwargs'])
    except Exception as e:
        result_exception = str(e)

    return jsonify(object_response(res_obj, result_exception, module_paths))

@app.route('/get_item', methods=['GET'])
def get_item():
    global objects

    result_exception = None
    req = request.json

    logger.info('/get_item')
    logger.info(req)

    try:
        base_obj = objects[req['id']]
        res_obj = base_obj[req['item']]
    except Exception as e:
        result_exception = str(e)

    return jsonify(object_response(res_obj, result_exception))

@app.route('/clear', methods=['POST', 'GET'])
def clear():
    global objects

    logger.info('/clear')

    sc = pyspark.SparkContext.getOrCreate()
    sc.stop()

    objects = {}

    return 'ok'

def run(*args, **kwargs):
    if 'debug' not in kwargs or ('debug' in kwargs and kwargs['debug'] == False):
        app.logger.removeHandler(default_handler)
        app.logger = logger

        logger.info('Starting pyspark proxy web server')

    app.run(*args, **kwargs)

if __name__ == '__main__':
    app.run(debug=True, use_reloader=False)



import unittest

from test_spark_context import SparkContextTestCase
from test_sqlcontext import SQLContextTestCase
from test_dataframe import DataFrameTestCase
from test_column import ColumnTestCase
from test_dataframe_reader import DataFrameReaderTestCase
from test_dataframe_writer import DataFrameWriterTestCase
from test_exceptions import ExceptionTestCase

if __name__ == '__main__':
    loader = unittest.TestLoader()
    tests = [
        loader.loadTestsFromTestCase(SparkContextTestCase),
        loader.loadTestsFromTestCase(SQLContextTestCase),
        loader.loadTestsFromTestCase(DataFrameTestCase),
        loader.loadTestsFromTestCase(ColumnTestCase),
        loader.loadTestsFromTestCase(DataFrameReaderTestCase),
        loader.loadTestsFromTestCase(DataFrameWriterTestCase),
        loader.loadTestsFromTestCase(ExceptionTestCase)
    ]
    suite = unittest.TestSuite(tests)

    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(suite)



from pyspark_proxy.sql.context import SQLContext
from pyspark_proxy.sql.dataframe import DataFrame
from pyspark_proxy.sql.readwriter import DataFrameReader, DataFrameWriter
from pyspark_proxy.sql.column import Column
from pyspark_proxy.sql.types import Row

__all__ = [
    'SQLContext',
    'DataFrame',
    'DataFrameReader', 'DataFrameWriter',
    'Column', 'Row'
]



import os
import unittest

from base_test_case import BaseTestCase

from pyspark_proxy.sql import SQLContext

class SQLContextTestCase(BaseTestCase):
    def setUp(self):
        self.data_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'data')

    def test_get_or_create(self):
        ctxt = SQLContext.getOrCreate(self.sc)

        self.assertEqual(ctxt.__class__, SQLContext)

        df = ctxt.read.parquet(os.path.join(self.data_path, 'data.parquet'))

        self.assertEqual(df.count(), 3)


if __name__ == '__main__':
    unittest.main()



from pyspark_proxy.proxy import Proxy

__all__ = ['DataFrameReader', 'DataFrameWriter']

class DataFrameReader(Proxy):
    _chain_functions = ['option', 'options', 'format', 'schema']

    def __init__(self, obj, prop):
        self._parent_obj = obj
        self._parent_prop = prop

        self._func_chain = [{'func': self._parent_prop}]

    def __getattr__(self, name):
        def method(*args, **kwargs):
            self._func_chain.append({'func': name, 'args': args, 'kwargs': kwargs})

            if name in self._chain_functions:
                return self
            else:
                return self._call_chain(self._parent_obj)

        return method

class DataFrameWriter(Proxy):
    _chain_functions = [
            'format', 'mode', 'option', 'options',
            'partitionBy', 'bucketBy', 'sortBy']

    def __init__(self, obj, prop):
        self._parent_obj = obj
        self._parent_prop = prop

        self._func_chain = [{'func': self._parent_prop}]

    def __getattr__(self, name):
        def method(*args, **kwargs):
            self._func_chain.append({'func': name, 'args': args, 'kwargs': kwargs})

            if name in self._chain_functions:
                return self
            else:
                return self._call_chain(self._parent_obj)

        return method



import os
import shutil
import unittest
import pandas

from base_test_case import BaseTestCase

from pyspark_proxy.sql.column import Column
from pyspark_proxy.server.capture import Capture

class ColumnTestCase(BaseTestCase):
    def setUp(self):
        self.current_path = os.path.abspath(os.path.dirname(__file__))
        self.output_path = os.path.join(self.current_path, 'test_output')

        data = [(1,2,'a'),(3,4,'b'),(5,6,'c')]
        self.df = self.sqlContext.createDataFrame(data, ['foo', 'bar', 'baz'])

    def tearDown(self):
        if os.path.exists(self.output_path):
            shutil.rmtree(self.output_path)

    def test_column_get_item(self):
        c = self.df['bar']

        self.assertEqual(type(c), Column)

    def test_column_get_attr(self):
        c = self.df.bar

        self.assertEqual(type(c), Column)

    def test_select_with_column(self):
        res_df = self.df.select(self.df['bar'], self.df.foo)

        self.assertEqual(res_df.columns, ['bar', 'foo'])

    def test_column_alias(self):
        res_df = self.df.select(self.df['foo'].alias('new_name'))

        self.assertEqual(res_df.columns, ['new_name'])

    def test_column_cast(self):
        res_df = self.df.select(self.df['foo'].cast('string'))

        with Capture() as output:
            res_df.printSchema()

        self.assertIn('string', '\n'.join(output))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import os
from distutils.core import setup

with open('requirements.txt') as fp:
    requires = fp.read().split('\n')

with open('LICENSE') as fp:
    license = fp.read()

with open('README.rst') as fp:
    readme = fp.read()

setup(
    name='PysparkProxy',
    version='0.0.10',
    packages=[
        'pyspark_proxy',
        'pyspark_proxy.server',
        'pyspark_proxy.sql'],
    license='Apache 2.0',
    description='Seamlessly execute pyspark code on remote clusters',
    long_description=readme,
    install_requires=requires,
    python_requires='>=2.7',
    url='https://github.com/abronte/PysparkProxy',
    scripts=['bin/pyspark-proxy-server'],
    author='Adam Bronte',
    author_email='adam@bronte.me',
)
